package com.mh.garrison.job;

import com.alibaba.druid.util.StringUtils;
import com.alibaba.fastjson.JSONObject;
import com.mh.garrison.constant.Constants;
import com.mh.garrison.mhutils.FastJson;
import com.mh.garrison.mhutils.StringUtil;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.nio.file.Files;

/**
 * 
 * @ClassName:  DealMeterLog   
 * @Description:TODO(定时处理电表数据)   
 * @author: 铭汉科技—LJF 
 * @date:   2020年7月20日 下午1:37:23   
 *
 */
@Component
public class NewDealMeterLogJob {

	// 5秒处理一次
	@Scheduled(cron = "0/5 * * * * ? ")
	public void dealMeterLogJob() {
		GetTxts(Constants.NEW_NB_METER_FILE);
	}

	public static void GetTxts(String filePath) {
		FastJson fastJson = new FastJson();
		File rootFile = new File(filePath);
		if (!rootFile.isDirectory()) {
			// System.out.println("文件名----" + rootFile.getAbsolutePath());
			// 读取文件内容
			try { // 防止文件建立或读取失败，用catch捕捉错误并打印，也可以throw
				/* 读入TXT文件 */
				String line;
				String jsonStr = null;
				
				String pathname = rootFile.getAbsolutePath(); // 绝对路径或相对路径都可以，写入文件时演示相对路径
				File filename = new File(pathname); // 要读取以上路径的input.txt文件
				if (filename.length()>0) {
					InputStreamReader reader = new InputStreamReader(Files.newInputStream(filename.toPath())); // 建立一个输入流对象reader
					BufferedReader br = new BufferedReader(reader); // 建立一个对象，它把文件内容转成计算机能读懂的语言
	
					// 网友推荐更加简洁的写法
					System.out.println("文件内容：" + br.lines());
	
					while ((line = br.readLine()) != null) {
						String payload = JSONObject.parseObject(line).getString("payload");
						if (!payload.isEmpty()){
							//走AEP平台解析
							jsonStr = fastJson.analysisEleFastJsonByAEPProfile(line);
						} else {
							//OC
							jsonStr = fastJson.analysisMeterFastJson(line);
						}
					}
					reader.close();
					br.close();
					if (!StringUtils.isEmpty(jsonStr)){
						if (jsonStr.equals("success")) {
							// 删除filename
							if (filename.exists()) {
								filename.delete();
								System.out.println("删除成功");
							}
						}
					}
				} else {
					// 删除filename
					if (filename.exists()) {
						filename.delete();
						System.out.println("删除成功");
					}
				}
			} catch (Exception e) {
				String causeStr = e.getCause().getMessage();
				System.out.println(">>>>>>>-------------" + causeStr);
			}
		} else {
			// 递归查询文件内容
			String[] fileList = rootFile.list();
			assert fileList != null;
			for (String s : fileList) {
				filePath = rootFile.getAbsolutePath() + "\\" + s;
				System.out.println("文件路径：" + filePath);
				try {
					GetTxts(filePath);
				} catch (Exception e) {
					// TODO: handle exception
					// 删除filename
					File rootFile1 = new File(filePath);
					if (rootFile1.exists()) {
						rootFile1.delete();
						System.out.println("出现异常，删除成功");
					}
					e.printStackTrace();
				}

			}
		}
	}
}
