package com.mh.garrison.job;

import java.io.IOException;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.mh.garrison.entity.AEPEntity;
import com.mh.garrison.mhutils.*;
import com.mh.garrison.service.MeterService;
import org.apache.http.HttpResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.mh.garrison.entity.OrderEntity;
import com.mh.garrison.service.OrderService;

/**
 * @ClassName: SendOrderJob
 * @Description:TODO(定时发送指令)
 * @author: 铭汉科技—LJF
 * @date: 2020年7月20日 上午9:08:28
 */
@Component
public class SendOrderJob {

    private final OrderService orderService;

    private final MeterService meterService;

    public SendOrderJob(OrderService orderService, MeterService meterService) {
        // TODO Auto-generated constructor stub
        this.orderService = orderService;
        this.meterService = meterService;
    }

    @Scheduled(cron = "0/30 * * * * ? ")
    public void sendOrder() {
        System.out.println("每30秒定时发送指令状态值");
        try {
            if (sendOrderJob()) {
                System.out.println("------------已发送报文-------------");
            } else {
                System.out.println("------------未发送报文-------------");
            }
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    private boolean sendOrderJob() {
        // TODO Auto-generated method stub
        /*
         * the device must connect to IoT platform before na post asyn command to device
         */
        // {
        // "_class" : "com.huawei.iom.iocm.domain.nsse.NsseRoute",
        // "deviceId" : "8c23b6b4-ea68-48fb-9c2f-90452a81ebb1",
        // "appId" : "pAw9x9zinQnQkYSLWbiGI_O6iBUa",
        // "nsseId" : "http://185.11.1.43:8096",
        // "edgeGwId" : "MeterCig",
        // "edgeGwType" : "CoAP",
        // "status" : "ONLINE"
        // }
//		Double test = 9.9;

        Boolean send_sms = false;
        try {
            List<OrderEntity> orderEntities = orderService.queryOrderList();
            if (orderEntities.size() != 0) {
                for (int i = 0; i < orderEntities.size(); i++) {
                    // 判断当前电表平台类型
                    OrderEntity orderEntity = orderEntities.get(i);
                    // 0:默认OC平台，1：AEP平台
                    Integer platformType = meterService.selectPlatformType(orderEntity.getHouseNum(), orderEntity.getDeviceNum());
                    if (platformType == 1) {
                        System.out.println("------------发送AEP平台-------------");
                        sendAEPOrder(orderEntity);
                        Thread.sleep(3000); // 发送命令之后延时3秒钟
                        send_sms = true;
                        continue;
                    }
                    // Two-Way Authentication
                    HttpsUtil httpsUtil = new HttpsUtil();
                    httpsUtil.initSSLConfigForTwoWay();

                    // Authentication，get token
                    String accessToken = login(httpsUtil);

                    // Please make sure that the following parameter values have been modified in
                    // the Constant file.
                    String urlPostAsynCmd = Constant.POST_ASYN_CMD;
                    String appId = Constant.APPID;

                    // please replace the deviceId, when you use the demo.
                    // String deviceId = "e86be9aa-d61b-428f-9d3f-f0dcef365308";
                    String deviceId = null;
                    try {
                        deviceId = orderEntities.get(i).getDeviceId();
                    } catch (Exception e) {
                        // TODO: handle exception
                        deviceId = "空值";
                    }


                    if (deviceId != null) {
                        String callbackUrl = Constant.REPORT_CMD_EXEC_RESULT_CALLBACK_URL;

                        // please replace the following parameter values, when you use the demo.
                        // And those parameter values must be consistent with the content of profile
                        // that have been preset to IoT platform.
                        // The following parameter values of this demo are use the watermeter profile
                        // that already initialized to IoT platform.
                        String serviceId = "MHMeterData";
                        String method = "COMMAND";
                        String sendOrder = orderEntities.get(i).getSendOrder();
                        // ObjectNode paras =
                        //        ObjectNode paras = JsonUtil.convertObject2ObjectNode("{\"value\":\"689999999999996804085BF33333333388667A16\"}");	// 拉闸
                        //        ObjectNode paras = JsonUtil.convertObject2ObjectNode("{\"value\":\"689999999999996804085BF33333333399CCF116\"}");   // 合闸
                        // JsonUtil.convertObject2ObjectNode("{\"value\":\"689999999999996804085BF33333333388667A16\"}");
                        // // 拉闸
                        ObjectNode paras = JsonUtil.convertObject2ObjectNode("{\"value\":\"" + sendOrder + "\"}"); // 合闸

                        Map<String, Object> paramCommand = new HashMap<String, Object>();
                        paramCommand.put("serviceId", serviceId);
                        paramCommand.put("method", method);
                        paramCommand.put("paras", paras);

                        Map<String, Object> paramPostAsynCmd = new HashMap<String, Object>();
                        paramPostAsynCmd.put("deviceId", deviceId);
                        paramPostAsynCmd.put("command", paramCommand);
                        paramPostAsynCmd.put("callbackUrl", callbackUrl); // 设置回调地址
                        paramPostAsynCmd.put("expireTime", "0"); // 设置发送时间

                        String jsonRequest = JsonUtil.jsonObj2Sting(paramPostAsynCmd);

                        Map<String, String> header = new HashMap<String, String>();
                        header.put(Constant.HEADER_APP_KEY, appId);
                        header.put(Constant.HEADER_APP_AUTH, "Bearer" + " " + accessToken);

                        HttpResponse responsePostAsynCmd = httpsUtil.doPostJson(urlPostAsynCmd, header, jsonRequest);

                        String responseBody = httpsUtil.getHttpResponseBody(responsePostAsynCmd);

                        System.out.println("PostAsynCommand, response content:");
                        System.out.print(responsePostAsynCmd.getStatusLine());
                        System.out.println(responseBody);
                        System.out.println();
                        Thread.sleep(3000); // 发送命令之后延时3秒钟
                        send_sms = true;
                    }
                }
            }
        } catch (IOException e) {
            // TODO: handle exception
            // e.printStackTrace();
            System.out.println("----------------" + e.getMessage().toString() + "---------------------");
            return send_sms;
        } catch (SQLException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (Throwable e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return send_sms;

    }

    private void sendAEPOrder(OrderEntity orderEntity) {
        try {
            String deviceId = orderEntity.getDeviceId();
            Integer groupId = 153097;
            Integer productNum = 17171214;
            String masterAPIkey = "72a2c789bb3f4f90b6e5bbffbe6e86b7";
            AEPEntity aepEntity = new AEPEntity();
            aepEntity.setDeviceGroupId(groupId);
            aepEntity.setMasterAPIKey(masterAPIkey);
            aepEntity.setProductId(productNum);
            PostCmdUtils.postAsyncCmdAEPV1ByProfile(deviceId, orderEntity.getSendOrder(), aepEntity);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("----------------" + e.getMessage() + "---------------------");
        }
    }

    /**
     * Authentication，get token
     */
    @SuppressWarnings("unchecked")
    public static String login(HttpsUtil httpsUtil) throws Exception {

        String appId = Constant.APPID;
        String secret = Constant.SECRET;
        String urlLogin = Constant.APP_AUTH;

        Map<String, String> paramLogin = new HashMap<String, String>();
        paramLogin.put("appId", appId);
        paramLogin.put("secret", secret);

        StreamClosedHttpResponse responseLogin = httpsUtil.doPostFormUrlEncodedGetStatusLine(urlLogin, paramLogin);

        System.out.println("app auth success,return accessToken:");
        System.out.print(responseLogin.getStatusLine());
        System.out.println(responseLogin.getContent());
        System.out.println();

        Map<String, String> data = new HashMap<String, String>();
        data = JsonUtil.jsonString2SimpleObj(responseLogin.getContent(), data.getClass());
        return data.get("accessToken");
    }
}
