package com.mh.garrison.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Options;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;
import org.apache.ibatis.mapping.StatementType;
import org.springframework.dao.DataAccessException;

import com.mh.garrison.entity.WaterDayBean;
import com.mh.garrison.entity.WaterHistoryBean;

@Mapper
public interface WaterDayMapper {
	
	@Select("select id,wt_num,true_flow,cur_date from wt_day where grade = 0 order by cur_date desc")
	List<WaterDayBean> findAllWaterDay();
	
	@Select ("select count(id) as tatol_num from wt_day where grade = 0")
	int findNotUploadNum();
	
	@Select("select id,wt_num,true_flow,cur_date from wt_day where wt_num = #{wt_num} and grade = 0 order by cur_date desc")
	List<WaterDayBean> findByWaterId(@Param("wt_num") String wt_num);
	
	/**
	 * 按照表号，时间查询
	 * @param wt_num
	 * @return
	 */
	@Select("select id,wt_num,true_flow,cur_date from wt_day where wt_num = #{wt_num} and convert(varchar(10), cur_date,121) = #{cur_date} and grade = 0 order by cur_date desc")
	List<WaterDayBean> findByDate(@Param("wt_num") String wt_num, @Param("cur_date") String dateString);
	
	/**
	 * 按照时间查询
	 * @param wt_num
	 * @return
	 */
	@Select("select id,wt_num,true_flow,cur_date from wt_day where convert(varchar(10), cur_date,121) = #{cur_date} and grade = 0 order by cur_date desc")
	List<WaterDayBean> findByTime(@Param("cur_date") String dateString);
	
	@Select("select top 1 COUNT(*) as sum1 from t_users_day where users_name = #{users_name} and cur_date = #{cur_date}")
	int checkSame(WaterHistoryBean waterDayBean);
	
	@Insert("insert into t_water_day(users_name) values (#{users_name})")
	int insert(@Param("users_name") String users_name);
	
	@Insert("insert into t_users_day(users_name, currentFlow, cur_date, year_date, month_date, grade)"
			+ " values (#{users_name}, #{currentFlow}, #{cur_date}, #{year_date}, #{month_date}, #{grade})")
	int insertAll(WaterHistoryBean waterDayBean) throws DataAccessException;
	
	@Update("update t_users_day set currentFlow=#{currentFlow} where users_name=#{users_name} and cur_date=#{cur_date}")
	int update(WaterHistoryBean waterDayBean);
		
	// 更新grade为1
	@Update("update wt_day set grade = 1 where wt_num = #{wt_num} and cur_date = #{cur_date}")
	boolean updateGrade(WaterDayBean waterDayBean);
	
	@Delete("delete from t_users_day where users_name=#{users_name}")
	int delete(String users_name);
	
	// 执行存储过程，计算出每天的用量
/*	@Select({"exec proc_calc_flow #{users_name,mode=IN,jdbcType=VARCHAR},"
			+ "#{date,mode=IN,jdbcType=VARCHAR}"})
	@Options(statementType=StatementType.CALLABLE)
	String exeProcedure1(WaterDayBean waterDayBean);
*/	
	@Select({"exec calculate_cursor #{users_name,mode=IN,jdbcType=VARCHAR},"
			+ "#{cur_date,mode=IN,jdbcType=VARCHAR}"})
	@Options(statementType=StatementType.CALLABLE)
	String exeProcedure(WaterHistoryBean waterDayBean);
	
	// 更新水表属于什么地方或者地区小区
	@Select({"exec pro_check_areas #{users_name,mode=IN,jdbcType=VARCHAR}"})
	@Options(statementType=StatementType.CALLABLE)
	String execPro_Cheack_Areas(@Param("users_name") String users_name);
	
	
	// 查询是否已计算
	@Select("Select calculateFlow from t_users_day where users_name = #{users_name} and cur_date = #{cur_date}")
	String checkCalc(WaterHistoryBean waterDayBean);

}
